function varargout = speech_prosody(varargin)
% SPEECH_PROSODY MATLAB code for speech_prosody.fig
%      SPEECH_PROSODY, by itself creates a new SPEECH_PROSODY or raises the 
%      existing singleton*.
%
%      H = SPEECH_PROSODY returns the handle to a new SPEECH_PROSODY or the 
%          handle to the existing singleton*.
%
%      SPEECH_PROSODY('CALLBACK',hObject,eventData,handles,...) calls the 
%      local function named CALLBACK in SPEECH_PROSODY.M with the given 
%      input arguments.
%
%      SPEECH_PROSODY('Property','Value',...)  creates a new SPEECH_PROSODY 
%      or raises the existing singleton*.  Starting from the left, property 
%      value pairs are applied to the GUI before speech_prosody_OpeningFcn 
%      gets called. An unrecognized property name or invalid value makes 
%      property application stop. All inputs are passed to 
%      speech_prosody_OpeningFcn via varargin.
%
%      *See GUI Options on GUIDE's Tools menu.  Choose "GUI allows only one
%      instance to run (singleton)".
%
% See also: GUIDE, GUIDATA, GUIHANDLES
% Edit the above text to modify the response to help speech_prosody
% Last Modified by GUIDE v2.5 16-Apr-2016 19:11:19

gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @speech_prosody_OpeningFcn, ...
                   'gui_OutputFcn',  @speech_prosody_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end

function speech_prosody_OpeningFcn(hObject, eventdata, handles, varargin)
handles.output    = hObject;
handles.Mtx       = struct();
handles.N = 0;
handles.num_feat  = 12;
handles.feat_lab  = {'F0_mean', 'F0_median', 'F0_std', 'F0_relStd',     ...
                     'F0_VR', 'F0_relVR', 'SEO_mean', 'SEO_median',     ...
                     'SEO_std', 'SEO_relStd', 'SEO_VR', 'SEO_relVR'};
                 
guidata(hObject, handles);

function varargout = speech_prosody_OutputFcn(hObject, eventdata, handles) 
varargout{1} = handles.output;

% =========================================================================
% MAIN PROGRAM
% 
% This program computes several conventional prosodic speech features and
% plots a waveform, fundamental frequency, and squared energy operator to
% visualize possible monopitch, monoloudness in speech -> dysprosody. The
% features are stored in a table for easy manipulation.
%
% speech features:
% 1. monopitch:
%    - mean of fundamental frequency
%    - standard deviation of fundamental frequency
%    - relative standard deviation of fundamental frequency
%    - variation range of fundamental frequency
%    - relative variation range of fundamental frequency
% 2. monoloudness:
%    - mean of squared energy operator
%    - standard deviation of squared energy operator
%    - relative standard deviation of squared energy operator
%    - variation range of squared energy operator
%    - relative variation range of squared energy operator
%
% For more information, see:
% Zoltan Galaz, et al., Prosodic analysis of neutral, stress-modified and 
% rhymed speech in patients with Parkinson's disease, Computer Methods 
% and Programs in Biomedicine, Volume 127, April 2016, Pages 301-317, 
% ISSN 0169-2607, http://dx.doi.org/10.1016/j.cmpb.2015.12.011.

addpath(genpath(pwd));

% -------------------------------------------------------------------------
% load_input_file_Callback(hObject, eventdata, handles)
% This function loads the input *.wav file, enables the buttons and also
% plots the waveform of the loaded file.
%
function load_input_file_Callback(hObject, eventdata, handles)
[file_name, path_name] = uigetfile('*.wav', 'Select *.wav file');
if (~isequal(file_name, 0))
    
    % Load the *.wav file and get samples and the sampling frequency
    handles.path_name = path_name;
    handles.file_name = file_name;
    handles.wav_name  = [path_name file_name];   
    [handles.x, handles.fs] = audioread(handles.wav_name);
    
    % Reset the axes
    cla(handles.axes1, 'reset');
    cla(handles.axes2, 'reset');
    cla(handles.axes3, 'reset');
    
    % Plot the waveform of the loaded *.wav file
    n = 0:length(handles.x) - 1;
    t = n/handles.fs;
    axes(handles.axes1);
    plot(t, handles.x, 'b');
    title(['Waveform:', '  ', handles.file_name]);
    grid on; 
    axis tight;
    
    % set the output string (path to the file)
    set(handles.selected_path, 'String', handles.wav_name);
    
    % Enable the buttons (play the file, compute prosodic features)  
    set(handles.compute_prosodic_features, 'Enable', 'on');
    set(handles.play_wav, 'Enable', 'on');
    set(handles.export_to_xlsx, 'Enable', 'on');
    set(handles.export_to_csv, 'Enable', 'on');

    % Update the handles
    guidata(hObject, handles);
end

% -------------------------------------------------------------------------
% compute_prosodic_features_Callback(hObject, eventdata, handles)
% This function computes the prosodic speech features of the actually
% loaded *.wav file. It also plots F0 and SEO estimates to visualize
% possible dysprosody.
% 
function compute_prosodic_features_Callback(hObject, eventdata, handles)
[F0, monopitch_features] = calc_monopitch_features(handles.x, handles.fs);
[SEO, monoloud_features] = calc_monoloud_features(handles.x, handles.fs);

if (~(isempty(F0)) || (isempty(SE)))
    
    % Plot the fundamental frequency estimation
    cla(handles.axes2, 'reset');
    axes(handles.axes2);
    plot(F0, 'r');
    title('F0 estimation');
    axis tight;
    grid on;

    % Plot the squared energy operator estimation
    cla(handles.axes3, 'reset');
    axes(handles.axes3);
    plot(SEO, 'g'); 
    title('SEO estimation');
    axis tight;
    grid on;

    % Increment the iterator (number of *.wav files)
    handles.N = handles.N + 1;

    % Save the filename of the *.wav file
    handles.Mtx(handles.N).file_name = handles.file_name;

    % Extract the monopitch features
    handles.Mtx(handles.N).F0_mean   = monopitch_features.F0_mean;
    handles.Mtx(handles.N).F0_median = monopitch_features.F0_median;
    handles.Mtx(handles.N).F0_Std    = monopitch_features.F0_Std;
    handles.Mtx(handles.N).F0_relStd = monopitch_features.F0_relStd;
    handles.Mtx(handles.N).F0_VR     = monopitch_features.F0_VR;
    handles.Mtx(handles.N).F0_relVR  = monopitch_features.F0_relVR;

    % Extract the monoloudness features
    handles.Mtx(handles.N).SEO_mean   = monoloud_features.SEO_mean;
    handles.Mtx(handles.N).SEO_median = monoloud_features.SEO_median;
    handles.Mtx(handles.N).SEO_Std    = monoloud_features.SEO_Std;
    handles.Mtx(handles.N).SEO_relStd = monoloud_features.SEO_relStd;
    handles.Mtx(handles.N).SEO_VR     = monoloud_features.SEO_VR;
    handles.Mtx(handles.N).SEO_relVR  = monoloud_features.SEO_relVR;

    % Prepare data for UITable
    data = cell(length(handles.Mtx) + 1, handles.num_feat + 1);
    data(1, 2:end) = handles.feat_lab;
    for i = 2:(length(handles.Mtx) + 1)
        data{i, 1}  = handles.Mtx(i - 1).file_name;
        data{i, 2}  = handles.Mtx(i - 1).F0_mean;
        data{i, 3}  = handles.Mtx(i - 1).F0_median;
        data{i, 4}  = handles.Mtx(i - 1).F0_Std;
        data{i, 5}  = handles.Mtx(i - 1).F0_relStd;
        data{i, 6}  = handles.Mtx(i - 1).F0_VR;
        data{i, 7}  = handles.Mtx(i - 1).F0_relVR;
        data{i, 8}  = handles.Mtx(i - 1).SEO_mean;
        data{i, 9}  = handles.Mtx(i - 1).SEO_median;
        data{i, 10} = handles.Mtx(i - 1).SEO_Std;
        data{i, 11} = handles.Mtx(i - 1).SEO_relStd;
        data{i, 12} = handles.Mtx(i - 1).SEO_VR;
        data{i, 13} = handles.Mtx(i - 1).SEO_relVR;
    end

    % Update the UITable
    set(handles.features_table, 'Data', data);

    % Update the handles
    guidata(hObject, handles);
end

% -------------------------------------------------------------------------
% play_wav_Callback(hObject, eventdata, handles)
% This function plays the actual *.wav file using the function: sound()
%
function play_wav_Callback(hObject, eventdata, handles)
sound(handles.x, handles.fs);

% -------------------------------------------------------------------------
% export_to_xlsx_Callback(hObject, eventdata, handles)
% This function exports the UITable into *.xlsx file
%
function export_to_xlsx_Callback(hObject, eventdata, handles)
xlswrite('prosodic_features.xlsx', get(handles.features_table, 'Data'));

% -------------------------------------------------------------------------
% export_to_xlsx_Callback(hObject, eventdata, handles)
% This function exports the UITable into *.csv file
%
function export_to_csv_Callback(hObject, eventdata, handles)
cell2csv(get(handles.features_table, 'Data'), 'prosodic_features.csv');

% -------------------------------------------------------------------------
function features_table_CellEditCallback(hObject, eventdata, handles)
    function features_table_CreateFcn(hObject, eventdata, handles)

% -------------------------------------------------------------------------
function selected_path_Callback(hObject, eventdata, handles)
    function selected_path_CreateFcn(hObject, eventdata, handles)
        if ispc && isequal(get(hObject,'BackgroundColor'), ...
                get(0,'defaultUicontrolBackgroundColor'))
            set(hObject,'BackgroundColor','white');
        end
